# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["CallRetrieveResponse"]


class CallRetrieveResponse(BaseModel):
    account_sid: Optional[str] = None
    """The id of the account the resource belongs to."""

    answered_by: Optional[Literal["human", "machine", "not_sure"]] = None
    """
    The value of the answering machine detection result, if this feature was enabled
    for the call.
    """

    caller_name: Optional[str] = None
    """Caller ID, if present."""

    date_created: Optional[str] = None
    """The timestamp of when the resource was created."""

    date_updated: Optional[str] = None
    """The timestamp of when the resource was last updated."""

    direction: Optional[Literal["inbound", "outbound"]] = None
    """The direction of this call."""

    duration: Optional[str] = None
    """The duration of this call, given in seconds."""

    end_time: Optional[str] = None
    """The end time of this call."""

    from_: Optional[str] = FieldInfo(alias="from", default=None)
    """The phone number or SIP address that made this call."""

    from_formatted: Optional[str] = None
    """The from number formatted for display."""

    price: Optional[str] = None
    """The price of this call, the currency is specified in the price_unit field.

    Only populated when the call cost feature is enabled for the account.
    """

    price_unit: Optional[str] = None
    """The unit in which the price is given."""

    sid: Optional[str] = None
    """The identifier of this call."""

    start_time: Optional[str] = None
    """The start time of this call."""

    status: Optional[Literal["ringing", "in-progress", "canceled", "completed", "failed", "busy", "no-answer"]] = None
    """The status of this call."""

    to: Optional[str] = None
    """The phone number or SIP address that received this call."""

    to_formatted: Optional[str] = None
    """The to number formatted for display."""

    uri: Optional[str] = None
    """The relative URI for this call."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from datetime import datetime
from typing_extensions import Literal

from ....._models import BaseModel

__all__ = ["PlanGetStepDetailsResponse", "Data"]


class Data(BaseModel):
    description: str

    run_id: str

    sequence: int

    status: Literal["pending", "in_progress", "completed", "skipped", "failed"]

    step_id: str

    completed_at: Optional[datetime] = None

    metadata: Optional[Dict[str, object]] = None

    parent_step_id: Optional[str] = None

    started_at: Optional[datetime] = None


class PlanGetStepDetailsResponse(BaseModel):
    data: Data



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .media_resource import MediaResource

__all__ = ["MediaRetrieveResponse"]


class MediaRetrieveResponse(BaseModel):
    data: Optional[MediaResource] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["LedgerBillingGroupReport"]


class LedgerBillingGroupReport(BaseModel):
    id: Optional[str] = None
    """Identifies the type of resource."""

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    organization_id: Optional[str] = None
    """Uniquely identifies the organization that owns the resource."""

    record_type: Optional[Literal["ledger_billing_group_report"]] = None
    """Identifies the type of the resource."""

    report_url: Optional[str] = None
    """External url of the ledger billing group report, if the status is complete"""

    status: Optional[Literal["pending", "complete", "failed", "deleted"]] = None
    """Status of the ledger billing group report"""

    updated_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["RecordingListParams", "Filter", "FilterCreatedAt"]


class RecordingListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[conference_id], filter[created_at][gte],
    filter[created_at][lte], filter[call_leg_id], filter[call_session_id],
    filter[from], filter[to], filter[connection_id], filter[sip_call_id]
    """

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]


class FilterCreatedAt(TypedDict, total=False):
    gte: str
    """Returns only recordings created later than or at given ISO 8601 datetime."""

    lte: str
    """Returns only recordings created earlier than or at given ISO 8601 datetime."""


_FilterReservedKeywords = TypedDict(
    "_FilterReservedKeywords",
    {
        "from": str,
    },
    total=False,
)


class Filter(_FilterReservedKeywords, total=False):
    """Consolidated filter parameter (deepObject style).

    Originally: filter[conference_id], filter[created_at][gte], filter[created_at][lte], filter[call_leg_id], filter[call_session_id], filter[from], filter[to], filter[connection_id], filter[sip_call_id]
    """

    call_leg_id: str
    """If present, recordings will be filtered to those with a matching call_leg_id."""

    call_session_id: str
    """
    If present, recordings will be filtered to those with a matching
    call_session_id.
    """

    conference_id: str
    """Returns only recordings associated with a given conference."""

    connection_id: str
    """
    If present, recordings will be filtered to those with a matching `connection_id`
    attribute (case-sensitive).
    """

    created_at: FilterCreatedAt

    sip_call_id: str
    """
    If present, recordings will be filtered to those with a matching `sip_call_id`
    attribute. Matching is case-sensitive
    """

    to: str
    """
    If present, recordings will be filtered to those with a matching `to` attribute
    (case-sensitive).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from telnyx.types.phone_numbers import (
    PhoneNumbersJob,
    JobRetrieveResponse,
    JobDeleteBatchResponse,
    JobUpdateBatchResponse,
    JobUpdateEmergencySettingsBatchResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestJobs:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        job = client.phone_numbers.jobs.retrieve(
            "id",
        )
        assert_matches_type(JobRetrieveResponse, job, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.phone_numbers.jobs.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        job = response.parse()
        assert_matches_type(JobRetrieveResponse, job, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.phone_numbers.jobs.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            job = response.parse()
            assert_matches_type(JobRetrieveResponse, job, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.phone_numbers.jobs.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        job = client.phone_numbers.jobs.list()
        assert_matches_type(SyncDefaultFlatPagination[PhoneNumbersJob], job, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        job = client.phone_numbers.jobs.list(
            filter={"type": "update_emergency_settings"},
            page_number=0,
            page_size=0,
            sort="created_at",
        )
        assert_matches_type(SyncDefaultFlatPagination[PhoneNumbersJob], job, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.phone_numbers.jobs.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        job = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[PhoneNumbersJob], job, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.phone_numbers.jobs.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            job = response.parse()
            assert_matches_type(SyncDefaultFlatPagination[PhoneNumbersJob], job, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete_batch(self, client: Telnyx) -> None:
        job = client.phone_numbers.jobs.delete_batch(
            phone_numbers=["+19705555098", "+19715555098", "32873127836"],
        )
        assert_matches_type(JobDeleteBatchResponse, job, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete_batch(self, client: Telnyx) -> None:
        response = client.phone_numbers.jobs.with_raw_response.delete_batch(
            phone_numbers=["+19705555098", "+19715555098", "32873127836"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        job = response.parse()
        assert_matches_type(JobDeleteBatchResponse, job, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete_batch(self, client: Telnyx) -> None:
        with client.phone_numbers.jobs.with_streaming_response.delete_batch(
            phone_numbers=["+19705555098", "+19715555098", "32873127836"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            job = response.parse()
            assert_matches_type(JobDeleteBatchResponse, job, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_batch(self, client: Telnyx) -> None:
        job = client.phone_numbers.jobs.update_batch(
            phone_numbers=["1583466971586889004", "+13127367254"],
        )
        assert_matches_type(JobUpdateBatchResponse, job, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_batch_with_all_params(self, client: Telnyx) -> None:
        job = client.phone_numbers.jobs.update_batch(
            phone_numbers=["1583466971586889004", "+13127367254"],
            filter={
                "billing_group_id": "62e4bf2e-c278-4282-b524-488d9c9c43b2",
                "connection_id": "1521916448077776306",
                "customer_reference": "customer_reference",
                "emergency_address_id": "9102160989215728032",
                "has_bundle": "has_bundle",
                "phone_number": "phone_number",
                "status": "active",
                "tag": "tag",
                "voice_connection_name": {
                    "contains": "test",
                    "ends_with": "test",
                    "eq": "test",
                    "starts_with": "test",
                },
                "voice_usage_payment_method": "channel",
            },
            billing_group_id="dc8e4d67-33a0-4cbb-af74-7b58f05bd494",
            connection_id="dc8e4d67-33a0-4cbb-af74-7b58f05bd494",
            customer_reference="customer-reference",
            deletion_lock_enabled=True,
            external_pin="123456",
            hd_voice_enabled=True,
            tags=["tag"],
            voice={
                "call_forwarding": {
                    "call_forwarding_enabled": True,
                    "forwarding_type": "always",
                    "forwards_to": "+13035559123",
                },
                "call_recording": {
                    "inbound_call_recording_channels": "single",
                    "inbound_call_recording_enabled": True,
                    "inbound_call_recording_format": "wav",
                },
                "caller_id_name_enabled": True,
                "cnam_listing": {
                    "cnam_listing_details": "example",
                    "cnam_listing_enabled": True,
                },
                "inbound_call_screening": "disabled",
                "media_features": {
                    "accept_any_rtp_packets_enabled": True,
                    "rtp_auto_adjust_enabled": True,
                    "t38_fax_gateway_enabled": True,
                },
                "tech_prefix_enabled": True,
                "translated_number": "+13035559999",
                "usage_payment_method": "pay-per-minute",
            },
        )
        assert_matches_type(JobUpdateBatchResponse, job, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update_batch(self, client: Telnyx) -> None:
        response = client.phone_numbers.jobs.with_raw_response.update_batch(
            phone_numbers=["1583466971586889004", "+13127367254"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        job = response.parse()
        assert_matches_type(JobUpdateBatchResponse, job, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update_batch(self, client: Telnyx) -> None:
        with client.phone_numbers.jobs.with_streaming_response.update_batch(
            phone_numbers=["1583466971586889004", "+13127367254"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            job = response.parse()
            assert_matches_type(JobUpdateBatchResponse, job, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_emergency_settings_batch(self, client: Telnyx) -> None:
        job = client.phone_numbers.jobs.update_emergency_settings_batch(
            emergency_enabled=True,
            phone_numbers=["+19705555098", "+19715555098", "32873127836"],
        )
        assert_matches_type(JobUpdateEmergencySettingsBatchResponse, job, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_emergency_settings_batch_with_all_params(self, client: Telnyx) -> None:
        job = client.phone_numbers.jobs.update_emergency_settings_batch(
            emergency_enabled=True,
            phone_numbers=["+19705555098", "+19715555098", "32873127836"],
            emergency_address_id="53829456729313",
        )
        assert_matches_type(JobUpdateEmergencySettingsBatchResponse, job, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update_emergency_settings_batch(self, client: Telnyx) -> None:
        response = client.phone_numbers.jobs.with_raw_response.update_emergency_settings_batch(
            emergency_enabled=True,
            phone_numbers=["+19705555098", "+19715555098", "32873127836"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        job = response.parse()
        assert_matches_type(JobUpdateEmergencySettingsBatchResponse, job, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update_emergency_settings_batch(self, client: Telnyx) -> None:
        with client.phone_numbers.jobs.with_streaming_response.update_emergency_settings_batch(
            emergency_enabled=True,
            phone_numbers=["+19705555098", "+19715555098", "32873127836"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            job = response.parse()
            assert_matches_type(JobUpdateEmergencySettingsBatchResponse, job, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncJobs:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        job = await async_client.phone_numbers.jobs.retrieve(
            "id",
        )
        assert_matches_type(JobRetrieveResponse, job, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.phone_numbers.jobs.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        job = await response.parse()
        assert_matches_type(JobRetrieveResponse, job, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.phone_numbers.jobs.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            job = await response.parse()
            assert_matches_type(JobRetrieveResponse, job, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.phone_numbers.jobs.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        job = await async_client.phone_numbers.jobs.list()
        assert_matches_type(AsyncDefaultFlatPagination[PhoneNumbersJob], job, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        job = await async_client.phone_numbers.jobs.list(
            filter={"type": "update_emergency_settings"},
            page_number=0,
            page_size=0,
            sort="created_at",
        )
        assert_matches_type(AsyncDefaultFlatPagination[PhoneNumbersJob], job, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.phone_numbers.jobs.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        job = await response.parse()
        assert_matches_type(AsyncDefaultFlatPagination[PhoneNumbersJob], job, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.phone_numbers.jobs.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            job = await response.parse()
            assert_matches_type(AsyncDefaultFlatPagination[PhoneNumbersJob], job, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete_batch(self, async_client: AsyncTelnyx) -> None:
        job = await async_client.phone_numbers.jobs.delete_batch(
            phone_numbers=["+19705555098", "+19715555098", "32873127836"],
        )
        assert_matches_type(JobDeleteBatchResponse, job, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete_batch(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.phone_numbers.jobs.with_raw_response.delete_batch(
            phone_numbers=["+19705555098", "+19715555098", "32873127836"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        job = await response.parse()
        assert_matches_type(JobDeleteBatchResponse, job, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete_batch(self, async_client: AsyncTelnyx) -> None:
        async with async_client.phone_numbers.jobs.with_streaming_response.delete_batch(
            phone_numbers=["+19705555098", "+19715555098", "32873127836"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            job = await response.parse()
            assert_matches_type(JobDeleteBatchResponse, job, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_batch(self, async_client: AsyncTelnyx) -> None:
        job = await async_client.phone_numbers.jobs.update_batch(
            phone_numbers=["1583466971586889004", "+13127367254"],
        )
        assert_matches_type(JobUpdateBatchResponse, job, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_batch_with_all_params(self, async_client: AsyncTelnyx) -> None:
        job = await async_client.phone_numbers.jobs.update_batch(
            phone_numbers=["1583466971586889004", "+13127367254"],
            filter={
                "billing_group_id": "62e4bf2e-c278-4282-b524-488d9c9c43b2",
                "connection_id": "1521916448077776306",
                "customer_reference": "customer_reference",
                "emergency_address_id": "9102160989215728032",
                "has_bundle": "has_bundle",
                "phone_number": "phone_number",
                "status": "active",
                "tag": "tag",
                "voice_connection_name": {
                    "contains": "test",
                    "ends_with": "test",
                    "eq": "test",
                    "starts_with": "test",
                },
                "voice_usage_payment_method": "channel",
            },
            billing_group_id="dc8e4d67-33a0-4cbb-af74-7b58f05bd494",
            connection_id="dc8e4d67-33a0-4cbb-af74-7b58f05bd494",
            customer_reference="customer-reference",
            deletion_lock_enabled=True,
            external_pin="123456",
            hd_voice_enabled=True,
            tags=["tag"],
            voice={
                "call_forwarding": {
                    "call_forwarding_enabled": True,
                    "forwarding_type": "always",
                    "forwards_to": "+13035559123",
                },
                "call_recording": {
                    "inbound_call_recording_channels": "single",
                    "inbound_call_recording_enabled": True,
                    "inbound_call_recording_format": "wav",
                },
                "caller_id_name_enabled": True,
                "cnam_listing": {
                    "cnam_listing_details": "example",
                    "cnam_listing_enabled": True,
                },
                "inbound_call_screening": "disabled",
                "media_features": {
                    "accept_any_rtp_packets_enabled": True,
                    "rtp_auto_adjust_enabled": True,
                    "t38_fax_gateway_enabled": True,
                },
                "tech_prefix_enabled": True,
                "translated_number": "+13035559999",
                "usage_payment_method": "pay-per-minute",
            },
        )
        assert_matches_type(JobUpdateBatchResponse, job, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update_batch(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.phone_numbers.jobs.with_raw_response.update_batch(
            phone_numbers=["1583466971586889004", "+13127367254"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        job = await response.parse()
        assert_matches_type(JobUpdateBatchResponse, job, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update_batch(self, async_client: AsyncTelnyx) -> None:
        async with async_client.phone_numbers.jobs.with_streaming_response.update_batch(
            phone_numbers=["1583466971586889004", "+13127367254"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            job = await response.parse()
            assert_matches_type(JobUpdateBatchResponse, job, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_emergency_settings_batch(self, async_client: AsyncTelnyx) -> None:
        job = await async_client.phone_numbers.jobs.update_emergency_settings_batch(
            emergency_enabled=True,
            phone_numbers=["+19705555098", "+19715555098", "32873127836"],
        )
        assert_matches_type(JobUpdateEmergencySettingsBatchResponse, job, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_emergency_settings_batch_with_all_params(self, async_client: AsyncTelnyx) -> None:
        job = await async_client.phone_numbers.jobs.update_emergency_settings_batch(
            emergency_enabled=True,
            phone_numbers=["+19705555098", "+19715555098", "32873127836"],
            emergency_address_id="53829456729313",
        )
        assert_matches_type(JobUpdateEmergencySettingsBatchResponse, job, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update_emergency_settings_batch(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.phone_numbers.jobs.with_raw_response.update_emergency_settings_batch(
            emergency_enabled=True,
            phone_numbers=["+19705555098", "+19715555098", "32873127836"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        job = await response.parse()
        assert_matches_type(JobUpdateEmergencySettingsBatchResponse, job, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update_emergency_settings_batch(self, async_client: AsyncTelnyx) -> None:
        async with async_client.phone_numbers.jobs.with_streaming_response.update_emergency_settings_batch(
            emergency_enabled=True,
            phone_numbers=["+19705555098", "+19715555098", "32873127836"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            job = await response.parse()
            assert_matches_type(JobUpdateEmergencySettingsBatchResponse, job, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel
from .wdr_report import WdrReport

__all__ = ["DetailRecordsReportListResponse"]


class DetailRecordsReportListResponse(BaseModel):
    data: Optional[List[WdrReport]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Mapping, cast

import httpx

from ..types import media_list_params, media_update_params, media_upload_params
from .._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from .._utils import extract_files, maybe_transform, deepcopy_minimal, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    to_custom_raw_response_wrapper,
    async_to_streamed_response_wrapper,
    to_custom_streamed_response_wrapper,
    async_to_custom_raw_response_wrapper,
    async_to_custom_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.media_list_response import MediaListResponse
from ..types.media_update_response import MediaUpdateResponse
from ..types.media_upload_response import MediaUploadResponse
from ..types.media_retrieve_response import MediaRetrieveResponse

__all__ = ["MediaResource", "AsyncMediaResource"]


class MediaResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> MediaResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return MediaResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MediaResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return MediaResourceWithStreamingResponse(self)

    def retrieve(
        self,
        media_name: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MediaRetrieveResponse:
        """
        Returns the information about a stored media file.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not media_name:
            raise ValueError(f"Expected a non-empty value for `media_name` but received {media_name!r}")
        return self._get(
            f"/media/{media_name}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MediaRetrieveResponse,
        )

    def update(
        self,
        media_name: str,
        *,
        media_url: str | Omit = omit,
        ttl_secs: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MediaUpdateResponse:
        """
        Updates a stored media file.

        Args:
          media_url: The URL where the media to be stored in Telnyx network is currently hosted. The
              maximum allowed size is 20 MB.

          ttl_secs: The number of seconds after which the media resource will be deleted, defaults
              to 2 days. The maximum allowed vale is 630720000, which translates to 20 years.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not media_name:
            raise ValueError(f"Expected a non-empty value for `media_name` but received {media_name!r}")
        body = deepcopy_minimal(
            {
                "media_url": media_url,
                "ttl_secs": ttl_secs,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["media"]])
        if files:
            # It should be noted that the actual Content-Type header that will be
            # sent to the server will contain a `boundary` parameter, e.g.
            # multipart/form-data; boundary=---abc--
            extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._put(
            f"/media/{media_name}",
            body=maybe_transform(body, media_update_params.MediaUpdateParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MediaUpdateResponse,
        )

    def list(
        self,
        *,
        filter: media_list_params.Filter | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MediaListResponse:
        """
        Returns a list of stored media files.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[content_type][]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/media",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"filter": filter}, media_list_params.MediaListParams),
            ),
            cast_to=MediaListResponse,
        )

    def delete(
        self,
        media_name: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Deletes a stored media file.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not media_name:
            raise ValueError(f"Expected a non-empty value for `media_name` but received {media_name!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/media/{media_name}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def download(
        self,
        media_name: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BinaryAPIResponse:
        """
        Downloads a stored media file.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not media_name:
            raise ValueError(f"Expected a non-empty value for `media_name` but received {media_name!r}")
        extra_headers = {"Accept": "application/octet-stream", **(extra_headers or {})}
        return self._get(
            f"/media/{media_name}/download",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BinaryAPIResponse,
        )

    def upload(
        self,
        *,
        media_url: str,
        media_name: str | Omit = omit,
        ttl_secs: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MediaUploadResponse:
        """
        Upload media file to Telnyx so it can be used with other Telnyx services

        Args:
          media_url: The URL where the media to be stored in Telnyx network is currently hosted. The
              maximum allowed size is 20 MB.

          media_name: The unique identifier of a file.

          ttl_secs: The number of seconds after which the media resource will be deleted, defaults
              to 2 days. The maximum allowed vale is 630720000, which translates to 20 years.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal(
            {
                "media_url": media_url,
                "media_name": media_name,
                "ttl_secs": ttl_secs,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["media"]])
        if files:
            # It should be noted that the actual Content-Type header that will be
            # sent to the server will contain a `boundary` parameter, e.g.
            # multipart/form-data; boundary=---abc--
            extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._post(
            "/media",
            body=maybe_transform(body, media_upload_params.MediaUploadParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MediaUploadResponse,
        )


class AsyncMediaResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncMediaResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMediaResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMediaResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncMediaResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        media_name: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MediaRetrieveResponse:
        """
        Returns the information about a stored media file.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not media_name:
            raise ValueError(f"Expected a non-empty value for `media_name` but received {media_name!r}")
        return await self._get(
            f"/media/{media_name}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MediaRetrieveResponse,
        )

    async def update(
        self,
        media_name: str,
        *,
        media_url: str | Omit = omit,
        ttl_secs: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MediaUpdateResponse:
        """
        Updates a stored media file.

        Args:
          media_url: The URL where the media to be stored in Telnyx network is currently hosted. The
              maximum allowed size is 20 MB.

          ttl_secs: The number of seconds after which the media resource will be deleted, defaults
              to 2 days. The maximum allowed vale is 630720000, which translates to 20 years.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not media_name:
            raise ValueError(f"Expected a non-empty value for `media_name` but received {media_name!r}")
        body = deepcopy_minimal(
            {
                "media_url": media_url,
                "ttl_secs": ttl_secs,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["media"]])
        if files:
            # It should be noted that the actual Content-Type header that will be
            # sent to the server will contain a `boundary` parameter, e.g.
            # multipart/form-data; boundary=---abc--
            extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._put(
            f"/media/{media_name}",
            body=await async_maybe_transform(body, media_update_params.MediaUpdateParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MediaUpdateResponse,
        )

    async def list(
        self,
        *,
        filter: media_list_params.Filter | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MediaListResponse:
        """
        Returns a list of stored media files.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[content_type][]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/media",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"filter": filter}, media_list_params.MediaListParams),
            ),
            cast_to=MediaListResponse,
        )

    async def delete(
        self,
        media_name: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Deletes a stored media file.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not media_name:
            raise ValueError(f"Expected a non-empty value for `media_name` but received {media_name!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/media/{media_name}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def download(
        self,
        media_name: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncBinaryAPIResponse:
        """
        Downloads a stored media file.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not media_name:
            raise ValueError(f"Expected a non-empty value for `media_name` but received {media_name!r}")
        extra_headers = {"Accept": "application/octet-stream", **(extra_headers or {})}
        return await self._get(
            f"/media/{media_name}/download",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AsyncBinaryAPIResponse,
        )

    async def upload(
        self,
        *,
        media_url: str,
        media_name: str | Omit = omit,
        ttl_secs: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MediaUploadResponse:
        """
        Upload media file to Telnyx so it can be used with other Telnyx services

        Args:
          media_url: The URL where the media to be stored in Telnyx network is currently hosted. The
              maximum allowed size is 20 MB.

          media_name: The unique identifier of a file.

          ttl_secs: The number of seconds after which the media resource will be deleted, defaults
              to 2 days. The maximum allowed vale is 630720000, which translates to 20 years.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal(
            {
                "media_url": media_url,
                "media_name": media_name,
                "ttl_secs": ttl_secs,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["media"]])
        if files:
            # It should be noted that the actual Content-Type header that will be
            # sent to the server will contain a `boundary` parameter, e.g.
            # multipart/form-data; boundary=---abc--
            extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._post(
            "/media",
            body=await async_maybe_transform(body, media_upload_params.MediaUploadParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MediaUploadResponse,
        )


class MediaResourceWithRawResponse:
    def __init__(self, media: MediaResource) -> None:
        self._media = media

        self.retrieve = to_raw_response_wrapper(
            media.retrieve,
        )
        self.update = to_raw_response_wrapper(
            media.update,
        )
        self.list = to_raw_response_wrapper(
            media.list,
        )
        self.delete = to_raw_response_wrapper(
            media.delete,
        )
        self.download = to_custom_raw_response_wrapper(
            media.download,
            BinaryAPIResponse,
        )
        self.upload = to_raw_response_wrapper(
            media.upload,
        )


class AsyncMediaResourceWithRawResponse:
    def __init__(self, media: AsyncMediaResource) -> None:
        self._media = media

        self.retrieve = async_to_raw_response_wrapper(
            media.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            media.update,
        )
        self.list = async_to_raw_response_wrapper(
            media.list,
        )
        self.delete = async_to_raw_response_wrapper(
            media.delete,
        )
        self.download = async_to_custom_raw_response_wrapper(
            media.download,
            AsyncBinaryAPIResponse,
        )
        self.upload = async_to_raw_response_wrapper(
            media.upload,
        )


class MediaResourceWithStreamingResponse:
    def __init__(self, media: MediaResource) -> None:
        self._media = media

        self.retrieve = to_streamed_response_wrapper(
            media.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            media.update,
        )
        self.list = to_streamed_response_wrapper(
            media.list,
        )
        self.delete = to_streamed_response_wrapper(
            media.delete,
        )
        self.download = to_custom_streamed_response_wrapper(
            media.download,
            StreamedBinaryAPIResponse,
        )
        self.upload = to_streamed_response_wrapper(
            media.upload,
        )


class AsyncMediaResourceWithStreamingResponse:
    def __init__(self, media: AsyncMediaResource) -> None:
        self._media = media

        self.retrieve = async_to_streamed_response_wrapper(
            media.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            media.update,
        )
        self.list = async_to_streamed_response_wrapper(
            media.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            media.delete,
        )
        self.download = async_to_custom_streamed_response_wrapper(
            media.download,
            AsyncStreamedBinaryAPIResponse,
        )
        self.upload = async_to_streamed_response_wrapper(
            media.upload,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import ledger_billing_group_report_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.ledger_billing_group_report_create_response import LedgerBillingGroupReportCreateResponse
from ..types.ledger_billing_group_report_retrieve_response import LedgerBillingGroupReportRetrieveResponse

__all__ = ["LedgerBillingGroupReportsResource", "AsyncLedgerBillingGroupReportsResource"]


class LedgerBillingGroupReportsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> LedgerBillingGroupReportsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return LedgerBillingGroupReportsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LedgerBillingGroupReportsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return LedgerBillingGroupReportsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        month: int | Omit = omit,
        year: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LedgerBillingGroupReportCreateResponse:
        """
        Create a ledger billing group report

        Args:
          month: Month of the ledger billing group report

          year: Year of the ledger billing group report

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/ledger_billing_group_reports",
            body=maybe_transform(
                {
                    "month": month,
                    "year": year,
                },
                ledger_billing_group_report_create_params.LedgerBillingGroupReportCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LedgerBillingGroupReportCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LedgerBillingGroupReportRetrieveResponse:
        """
        Get a ledger billing group report

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/ledger_billing_group_reports/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LedgerBillingGroupReportRetrieveResponse,
        )


class AsyncLedgerBillingGroupReportsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncLedgerBillingGroupReportsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLedgerBillingGroupReportsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLedgerBillingGroupReportsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncLedgerBillingGroupReportsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        month: int | Omit = omit,
        year: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LedgerBillingGroupReportCreateResponse:
        """
        Create a ledger billing group report

        Args:
          month: Month of the ledger billing group report

          year: Year of the ledger billing group report

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/ledger_billing_group_reports",
            body=await async_maybe_transform(
                {
                    "month": month,
                    "year": year,
                },
                ledger_billing_group_report_create_params.LedgerBillingGroupReportCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LedgerBillingGroupReportCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LedgerBillingGroupReportRetrieveResponse:
        """
        Get a ledger billing group report

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/ledger_billing_group_reports/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LedgerBillingGroupReportRetrieveResponse,
        )


class LedgerBillingGroupReportsResourceWithRawResponse:
    def __init__(self, ledger_billing_group_reports: LedgerBillingGroupReportsResource) -> None:
        self._ledger_billing_group_reports = ledger_billing_group_reports

        self.create = to_raw_response_wrapper(
            ledger_billing_group_reports.create,
        )
        self.retrieve = to_raw_response_wrapper(
            ledger_billing_group_reports.retrieve,
        )


class AsyncLedgerBillingGroupReportsResourceWithRawResponse:
    def __init__(self, ledger_billing_group_reports: AsyncLedgerBillingGroupReportsResource) -> None:
        self._ledger_billing_group_reports = ledger_billing_group_reports

        self.create = async_to_raw_response_wrapper(
            ledger_billing_group_reports.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            ledger_billing_group_reports.retrieve,
        )


class LedgerBillingGroupReportsResourceWithStreamingResponse:
    def __init__(self, ledger_billing_group_reports: LedgerBillingGroupReportsResource) -> None:
        self._ledger_billing_group_reports = ledger_billing_group_reports

        self.create = to_streamed_response_wrapper(
            ledger_billing_group_reports.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            ledger_billing_group_reports.retrieve,
        )


class AsyncLedgerBillingGroupReportsResourceWithStreamingResponse:
    def __init__(self, ledger_billing_group_reports: AsyncLedgerBillingGroupReportsResource) -> None:
        self._ledger_billing_group_reports = ledger_billing_group_reports

        self.create = async_to_streamed_response_wrapper(
            ledger_billing_group_reports.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            ledger_billing_group_reports.retrieve,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ..._models import BaseModel
from .user_bundle import UserBundle

__all__ = ["UserBundleCreateResponse"]


class UserBundleCreateResponse(BaseModel):
    data: List[UserBundle]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["TelnyxConversationChannel"]

TelnyxConversationChannel: TypeAlias = Literal["phone_call", "web_call", "sms_chat", "web_chat"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from telnyx.types.ai.missions.runs import (
    EventLogResponse,
    EventListResponse,
    EventGetEventDetailsResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestEvents:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        event = client.ai.missions.runs.events.list(
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(SyncDefaultFlatPagination[EventListResponse], event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        event = client.ai.missions.runs.events.list(
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            agent_id="agent_id",
            page_number=1,
            page_size=1,
            step_id="step_id",
            type="type",
        )
        assert_matches_type(SyncDefaultFlatPagination[EventListResponse], event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.ai.missions.runs.events.with_raw_response.list(
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        event = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[EventListResponse], event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.ai.missions.runs.events.with_streaming_response.list(
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            event = response.parse()
            assert_matches_type(SyncDefaultFlatPagination[EventListResponse], event, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_list(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `mission_id` but received ''"):
            client.ai.missions.runs.events.with_raw_response.list(
                run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                mission_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `run_id` but received ''"):
            client.ai.missions.runs.events.with_raw_response.list(
                run_id="",
                mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get_event_details(self, client: Telnyx) -> None:
        event = client.ai.missions.runs.events.get_event_details(
            event_id="event_id",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(EventGetEventDetailsResponse, event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_get_event_details(self, client: Telnyx) -> None:
        response = client.ai.missions.runs.events.with_raw_response.get_event_details(
            event_id="event_id",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        event = response.parse()
        assert_matches_type(EventGetEventDetailsResponse, event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_get_event_details(self, client: Telnyx) -> None:
        with client.ai.missions.runs.events.with_streaming_response.get_event_details(
            event_id="event_id",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            event = response.parse()
            assert_matches_type(EventGetEventDetailsResponse, event, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_get_event_details(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `mission_id` but received ''"):
            client.ai.missions.runs.events.with_raw_response.get_event_details(
                event_id="event_id",
                mission_id="",
                run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `run_id` but received ''"):
            client.ai.missions.runs.events.with_raw_response.get_event_details(
                event_id="event_id",
                mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                run_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `event_id` but received ''"):
            client.ai.missions.runs.events.with_raw_response.get_event_details(
                event_id="",
                mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_log(self, client: Telnyx) -> None:
        event = client.ai.missions.runs.events.log(
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            summary="summary",
            type="status_change",
        )
        assert_matches_type(EventLogResponse, event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_log_with_all_params(self, client: Telnyx) -> None:
        event = client.ai.missions.runs.events.log(
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            summary="summary",
            type="status_change",
            agent_id="agent_id",
            idempotency_key="idempotency_key",
            payload={"foo": "bar"},
            step_id="step_id",
        )
        assert_matches_type(EventLogResponse, event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_log(self, client: Telnyx) -> None:
        response = client.ai.missions.runs.events.with_raw_response.log(
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            summary="summary",
            type="status_change",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        event = response.parse()
        assert_matches_type(EventLogResponse, event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_log(self, client: Telnyx) -> None:
        with client.ai.missions.runs.events.with_streaming_response.log(
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            summary="summary",
            type="status_change",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            event = response.parse()
            assert_matches_type(EventLogResponse, event, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_log(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `mission_id` but received ''"):
            client.ai.missions.runs.events.with_raw_response.log(
                run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                mission_id="",
                summary="summary",
                type="status_change",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `run_id` but received ''"):
            client.ai.missions.runs.events.with_raw_response.log(
                run_id="",
                mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                summary="summary",
                type="status_change",
            )


class TestAsyncEvents:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        event = await async_client.ai.missions.runs.events.list(
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(AsyncDefaultFlatPagination[EventListResponse], event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        event = await async_client.ai.missions.runs.events.list(
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            agent_id="agent_id",
            page_number=1,
            page_size=1,
            step_id="step_id",
            type="type",
        )
        assert_matches_type(AsyncDefaultFlatPagination[EventListResponse], event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.missions.runs.events.with_raw_response.list(
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        event = await response.parse()
        assert_matches_type(AsyncDefaultFlatPagination[EventListResponse], event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.missions.runs.events.with_streaming_response.list(
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            event = await response.parse()
            assert_matches_type(AsyncDefaultFlatPagination[EventListResponse], event, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_list(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `mission_id` but received ''"):
            await async_client.ai.missions.runs.events.with_raw_response.list(
                run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                mission_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `run_id` but received ''"):
            await async_client.ai.missions.runs.events.with_raw_response.list(
                run_id="",
                mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get_event_details(self, async_client: AsyncTelnyx) -> None:
        event = await async_client.ai.missions.runs.events.get_event_details(
            event_id="event_id",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(EventGetEventDetailsResponse, event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_get_event_details(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.missions.runs.events.with_raw_response.get_event_details(
            event_id="event_id",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        event = await response.parse()
        assert_matches_type(EventGetEventDetailsResponse, event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_get_event_details(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.missions.runs.events.with_streaming_response.get_event_details(
            event_id="event_id",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            event = await response.parse()
            assert_matches_type(EventGetEventDetailsResponse, event, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_get_event_details(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `mission_id` but received ''"):
            await async_client.ai.missions.runs.events.with_raw_response.get_event_details(
                event_id="event_id",
                mission_id="",
                run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `run_id` but received ''"):
            await async_client.ai.missions.runs.events.with_raw_response.get_event_details(
                event_id="event_id",
                mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                run_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `event_id` but received ''"):
            await async_client.ai.missions.runs.events.with_raw_response.get_event_details(
                event_id="",
                mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_log(self, async_client: AsyncTelnyx) -> None:
        event = await async_client.ai.missions.runs.events.log(
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            summary="summary",
            type="status_change",
        )
        assert_matches_type(EventLogResponse, event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_log_with_all_params(self, async_client: AsyncTelnyx) -> None:
        event = await async_client.ai.missions.runs.events.log(
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            summary="summary",
            type="status_change",
            agent_id="agent_id",
            idempotency_key="idempotency_key",
            payload={"foo": "bar"},
            step_id="step_id",
        )
        assert_matches_type(EventLogResponse, event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_log(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.missions.runs.events.with_raw_response.log(
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            summary="summary",
            type="status_change",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        event = await response.parse()
        assert_matches_type(EventLogResponse, event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_log(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.missions.runs.events.with_streaming_response.log(
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            summary="summary",
            type="status_change",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            event = await response.parse()
            assert_matches_type(EventLogResponse, event, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_log(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `mission_id` but received ''"):
            await async_client.ai.missions.runs.events.with_raw_response.log(
                run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                mission_id="",
                summary="summary",
                type="status_change",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `run_id` but received ''"):
            await async_client.ai.missions.runs.events.with_raw_response.log(
                run_id="",
                mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                summary="summary",
                type="status_change",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["PortingPhoneNumberListResponse"]


class PortingPhoneNumberListResponse(BaseModel):
    activation_status: Optional[
        Literal[
            "New",
            "Pending",
            "Conflict",
            "Cancel Pending",
            "Failed",
            "Concurred",
            "Activate RDY",
            "Disconnect Pending",
            "Concurrence Sent",
            "Old",
            "Sending",
            "Active",
            "Cancelled",
        ]
    ] = None
    """Activation status"""

    phone_number: Optional[str] = None
    """E164 formatted phone number"""

    phone_number_type: Optional[Literal["landline", "local", "mobile", "national", "shared_cost", "toll_free"]] = None
    """The type of the phone number"""

    portability_status: Optional[Literal["pending", "confirmed", "provisional"]] = None
    """
    Specifies whether Telnyx is able to confirm portability this number in the
    United States & Canada. International phone numbers are provisional by default.
    """

    porting_order_id: Optional[str] = None
    """Identifies the associated port request"""

    porting_order_status: Optional[
        Literal[
            "draft",
            "in-process",
            "submitted",
            "exception",
            "foc-date-confirmed",
            "cancel-pending",
            "ported",
            "cancelled",
        ]
    ] = None
    """The current status of the porting order"""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    requirements_status: Optional[
        Literal["requirement-info-pending", "requirement-info-under-review", "requirement-info-exception", "approved"]
    ] = None
    """The current status of the requirements in a INTL porting order"""

    support_key: Optional[str] = None
    """A key to reference this porting order when contacting Telnyx customer support"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional

from .record import Record
from .._models import BaseModel

__all__ = ["GlobalIPHealthCheckCreateResponse", "Data"]


class Data(Record):
    global_ip_id: Optional[str] = None
    """Global IP ID."""

    health_check_params: Optional[Dict[str, object]] = None
    """A Global IP health check params."""

    health_check_type: Optional[str] = None
    """The Global IP health check type."""


class GlobalIPHealthCheckCreateResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["Fax"]


class Fax(BaseModel):
    id: Optional[str] = None
    """Identifies the type of resource."""

    client_state: Optional[str] = None
    """State received from a command."""

    connection_id: Optional[str] = None
    """The ID of the connection used to send the fax."""

    created_at: Optional[datetime] = None
    """ISO 8601 timestamp when resource was created"""

    direction: Optional[Literal["inbound", "outbound"]] = None
    """The direction of the fax."""

    from_: Optional[str] = FieldInfo(alias="from", default=None)
    """The phone number, in E.164 format, the fax will be sent from."""

    from_display_name: Optional[str] = None
    """
    The string used as the caller id name (SIP From Display Name) presented to the
    destination (`to` number).
    """

    media_name: Optional[str] = None
    """The media_name used for the fax's media.

    Must point to a file previously uploaded to api.telnyx.com/v2/media by the same
    user/organization. media_name and media_url/contents can't be submitted
    together.
    """

    media_url: Optional[str] = None
    """The URL (or list of URLs) to the PDF used for the fax's media.

    media_url and media_name/contents can't be submitted together.
    """

    preview_url: Optional[str] = None
    """If `store_preview` was set to `true`, this is a link to temporary location.

    Link expires after 10 minutes.
    """

    quality: Optional[Literal["normal", "high", "very_high", "ultra_light", "ultra_dark"]] = None
    """The quality of the fax.

    The `ultra` settings provides the highest quality available, but also present
    longer fax processing times. `ultra_light` is best suited for images, wihle
    `ultra_dark` is best suited for text.
    """

    record_type: Optional[Literal["fax"]] = None
    """Identifies the type of the resource."""

    status: Optional[
        Literal[
            "queued",
            "media.processed",
            "originated",
            "sending",
            "delivered",
            "failed",
            "initiated",
            "receiving",
            "media.processing",
            "received",
        ]
    ] = None
    """Status of the fax"""

    store_media: Optional[bool] = None
    """Should fax media be stored on temporary URL. It does not support media_name."""

    stored_media_url: Optional[str] = None
    """If store_media was set to true, this is a link to temporary location.

    Link expires after 10 minutes.
    """

    to: Optional[str] = None
    """The phone number, in E.164 format, the fax will be sent to or SIP URI"""

    updated_at: Optional[datetime] = None
    """ISO 8601 timestamp when resource was updated"""

    webhook_failover_url: Optional[str] = None
    """
    Optional failover URL that will receive fax webhooks if webhook_url doesn't
    return a 2XX response
    """

    webhook_url: Optional[str] = None
    """URL that will receive fax webhooks"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["OAuthTokenResponse"]


class OAuthTokenResponse(BaseModel):
    access_token: str
    """The access token"""

    expires_in: int
    """Token lifetime in seconds"""

    token_type: Literal["Bearer"]
    """Token type"""

    refresh_token: Optional[str] = None
    """Refresh token (if applicable)"""

    scope: Optional[str] = None
    """Space-separated list of granted scopes"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .external_connection import ExternalConnection

__all__ = ["ExternalConnectionUpdateResponse"]


class ExternalConnectionUpdateResponse(BaseModel):
    data: Optional[ExternalConnection] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .call_refer_failed import CallReferFailed

__all__ = ["CallReferFailedWebhookEvent"]


class CallReferFailedWebhookEvent(BaseModel):
    data: Optional[CallReferFailed] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union
from datetime import date
from typing_extensions import Literal, Annotated, TypeAlias

from .._utils import PropertyInfo
from .._models import BaseModel
from .month_detail import MonthDetail

__all__ = [
    "ChargesSummaryRetrieveResponse",
    "Data",
    "DataSummary",
    "DataSummaryAdjustment",
    "DataSummaryLine",
    "DataSummaryLineComparative",
    "DataSummaryLineSimple",
    "DataTotal",
]


class DataSummaryAdjustment(BaseModel):
    amount: str
    """Adjustment amount as decimal string"""

    description: str
    """Description of the adjustment"""

    event_date: date
    """Date when the adjustment occurred"""


class DataSummaryLineComparative(BaseModel):
    alias: str
    """Service alias"""

    existing_this_month: MonthDetail

    name: str
    """Service name"""

    new_this_month: MonthDetail

    type: Literal["comparative"]


class DataSummaryLineSimple(BaseModel):
    alias: str
    """Service alias"""

    amount: str
    """Total amount as decimal string"""

    name: str
    """Service name"""

    quantity: int
    """Number of items"""

    type: Literal["simple"]


DataSummaryLine: TypeAlias = Annotated[
    Union[DataSummaryLineComparative, DataSummaryLineSimple], PropertyInfo(discriminator="type")
]


class DataSummary(BaseModel):
    adjustments: List[DataSummaryAdjustment]
    """List of billing adjustments"""

    lines: List[DataSummaryLine]
    """List of charge summary lines"""


class DataTotal(BaseModel):
    credits: str
    """Total credits as decimal string"""

    existing_mrc: str
    """Total existing monthly recurring charges as decimal string"""

    grand_total: str
    """Grand total of all charges as decimal string"""

    ledger_adjustments: str
    """Ledger adjustments as decimal string"""

    new_mrc: str
    """Total new monthly recurring charges as decimal string"""

    new_otc: str
    """Total new one-time charges as decimal string"""

    other: str
    """Other charges as decimal string"""


class Data(BaseModel):
    currency: str
    """Currency code"""

    end_date: date
    """End date of the summary period"""

    start_date: date
    """Start date of the summary period"""

    summary: DataSummary

    total: DataTotal

    user_email: str
    """User email address"""

    user_id: str
    """User identifier"""


class ChargesSummaryRetrieveResponse(BaseModel):
    data: Data



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .shared.room_participant import RoomParticipant

__all__ = ["RoomParticipantRetrieveResponse"]


class RoomParticipantRetrieveResponse(BaseModel):
    data: Optional[RoomParticipant] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .authentication_provider import AuthenticationProvider

__all__ = ["AuthenticationProviderRetrieveResponse"]


class AuthenticationProviderRetrieveResponse(BaseModel):
    data: Optional[AuthenticationProvider] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ...types import (
    managed_account_list_params,
    managed_account_create_params,
    managed_account_update_params,
    managed_account_update_global_channel_limit_params,
)
from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from ..._base_client import AsyncPaginator, make_request_options
from ...types.managed_account_list_response import ManagedAccountListResponse
from ...types.managed_account_create_response import ManagedAccountCreateResponse
from ...types.managed_account_update_response import ManagedAccountUpdateResponse
from ...types.managed_account_retrieve_response import ManagedAccountRetrieveResponse
from ...types.managed_account_update_global_channel_limit_response import ManagedAccountUpdateGlobalChannelLimitResponse
from ...types.managed_account_get_allocatable_global_outbound_channels_response import (
    ManagedAccountGetAllocatableGlobalOutboundChannelsResponse,
)

__all__ = ["ManagedAccountsResource", "AsyncManagedAccountsResource"]


class ManagedAccountsResource(SyncAPIResource):
    @cached_property
    def actions(self) -> ActionsResource:
        return ActionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> ManagedAccountsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return ManagedAccountsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ManagedAccountsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return ManagedAccountsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        business_name: str,
        email: str | Omit = omit,
        managed_account_allow_custom_pricing: bool | Omit = omit,
        password: str | Omit = omit,
        rollup_billing: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ManagedAccountCreateResponse:
        """Create a new managed account owned by the authenticated user.

        You need to be
        explictly approved by Telnyx in order to become a manager account.

        Args:
          business_name: The name of the business for which the new managed account is being created,
              that will be used as the managed accounts's organization's name.

          email: The email address for the managed account. If not provided, the email address
              will be generated based on the email address of the manager account.

          managed_account_allow_custom_pricing: Boolean value that indicates if the managed account is able to have custom
              pricing set for it or not. If false, uses the pricing of the manager account.
              Defaults to false. This value may be changed after creation, but there may be
              time lag between when the value is changed and pricing changes take effect.

          password: Password for the managed account. If a password is not supplied, the account
              will not be able to be signed into directly. (A password reset may still be
              performed later to enable sign-in via password.)

          rollup_billing: Boolean value that indicates if the billing information and charges to the
              managed account "roll up" to the manager account. If true, the managed account
              will not have its own balance and will use the shared balance with the manager
              account. This value cannot be changed after account creation without going
              through Telnyx support as changes require manual updates to the account ledger.
              Defaults to false.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/managed_accounts",
            body=maybe_transform(
                {
                    "business_name": business_name,
                    "email": email,
                    "managed_account_allow_custom_pricing": managed_account_allow_custom_pricing,
                    "password": password,
                    "rollup_billing": rollup_billing,
                },
                managed_account_create_params.ManagedAccountCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ManagedAccountCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ManagedAccountRetrieveResponse:
        """
        Retrieves the details of a single managed account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/managed_accounts/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ManagedAccountRetrieveResponse,
        )

    def update(
        self,
        id: str,
        *,
        managed_account_allow_custom_pricing: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ManagedAccountUpdateResponse:
        """
        Update a single managed account.

        Args:
          managed_account_allow_custom_pricing: Boolean value that indicates if the managed account is able to have custom
              pricing set for it or not. If false, uses the pricing of the manager account.
              Defaults to false. This value may be changed, but there may be time lag between
              when the value is changed and pricing changes take effect.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/managed_accounts/{id}",
            body=maybe_transform(
                {"managed_account_allow_custom_pricing": managed_account_allow_custom_pricing},
                managed_account_update_params.ManagedAccountUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ManagedAccountUpdateResponse,
        )

    def list(
        self,
        *,
        filter: managed_account_list_params.Filter | Omit = omit,
        include_cancelled_accounts: bool | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: Literal["created_at", "email"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[ManagedAccountListResponse]:
        """Lists the accounts managed by the current user.

        Users need to be explictly
        approved by Telnyx in order to become manager accounts.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[email][contains], filter[email][eq], filter[organization_name][contains],
              filter[organization_name][eq]

          include_cancelled_accounts: Specifies if cancelled accounts should be included in the results.

          sort: Specifies the sort order for results. By default sorting direction is ascending.
              To have the results sorted in descending order add the <code> -</code>
              prefix.<br/><br/> That is: <ul>

                <li>
                  <code>email</code>: sorts the result by the
                  <code>email</code> field in ascending order.
                </li>

                <li>
                  <code>-email</code>: sorts the result by the
                  <code>email</code> field in descending order.
                </li>
              </ul> <br/> If not given, results are sorted by <code>created_at</code> in descending order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/managed_accounts",
            page=SyncDefaultFlatPagination[ManagedAccountListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "include_cancelled_accounts": include_cancelled_accounts,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    managed_account_list_params.ManagedAccountListParams,
                ),
            ),
            model=ManagedAccountListResponse,
        )

    def get_allocatable_global_outbound_channels(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ManagedAccountGetAllocatableGlobalOutboundChannelsResponse:
        """
        Display information about allocatable global outbound channels for the current
        user. Only usable by account managers.
        """
        return self._get(
            "/managed_accounts/allocatable_global_outbound_channels",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ManagedAccountGetAllocatableGlobalOutboundChannelsResponse,
        )

    def update_global_channel_limit(
        self,
        id: str,
        *,
        channel_limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ManagedAccountUpdateGlobalChannelLimitResponse:
        """
        Update the amount of allocatable global outbound channels allocated to a
        specific managed account.

        Args:
          channel_limit: Integer value that indicates the number of allocatable global outbound channels
              that should be allocated to the managed account. Must be 0 or more. If the value
              is 0 then the account will have no usable channels and will not be able to
              perform outbound calling.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/managed_accounts/{id}/update_global_channel_limit",
            body=maybe_transform(
                {"channel_limit": channel_limit},
                managed_account_update_global_channel_limit_params.ManagedAccountUpdateGlobalChannelLimitParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ManagedAccountUpdateGlobalChannelLimitResponse,
        )


class AsyncManagedAccountsResource(AsyncAPIResource):
    @cached_property
    def actions(self) -> AsyncActionsResource:
        return AsyncActionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncManagedAccountsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncManagedAccountsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncManagedAccountsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncManagedAccountsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        business_name: str,
        email: str | Omit = omit,
        managed_account_allow_custom_pricing: bool | Omit = omit,
        password: str | Omit = omit,
        rollup_billing: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ManagedAccountCreateResponse:
        """Create a new managed account owned by the authenticated user.

        You need to be
        explictly approved by Telnyx in order to become a manager account.

        Args:
          business_name: The name of the business for which the new managed account is being created,
              that will be used as the managed accounts's organization's name.

          email: The email address for the managed account. If not provided, the email address
              will be generated based on the email address of the manager account.

          managed_account_allow_custom_pricing: Boolean value that indicates if the managed account is able to have custom
              pricing set for it or not. If false, uses the pricing of the manager account.
              Defaults to false. This value may be changed after creation, but there may be
              time lag between when the value is changed and pricing changes take effect.

          password: Password for the managed account. If a password is not supplied, the account
              will not be able to be signed into directly. (A password reset may still be
              performed later to enable sign-in via password.)

          rollup_billing: Boolean value that indicates if the billing information and charges to the
              managed account "roll up" to the manager account. If true, the managed account
              will not have its own balance and will use the shared balance with the manager
              account. This value cannot be changed after account creation without going
              through Telnyx support as changes require manual updates to the account ledger.
              Defaults to false.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/managed_accounts",
            body=await async_maybe_transform(
                {
                    "business_name": business_name,
                    "email": email,
                    "managed_account_allow_custom_pricing": managed_account_allow_custom_pricing,
                    "password": password,
                    "rollup_billing": rollup_billing,
                },
                managed_account_create_params.ManagedAccountCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ManagedAccountCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ManagedAccountRetrieveResponse:
        """
        Retrieves the details of a single managed account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/managed_accounts/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ManagedAccountRetrieveResponse,
        )

    async def update(
        self,
        id: str,
        *,
        managed_account_allow_custom_pricing: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ManagedAccountUpdateResponse:
        """
        Update a single managed account.

        Args:
          managed_account_allow_custom_pricing: Boolean value that indicates if the managed account is able to have custom
              pricing set for it or not. If false, uses the pricing of the manager account.
              Defaults to false. This value may be changed, but there may be time lag between
              when the value is changed and pricing changes take effect.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/managed_accounts/{id}",
            body=await async_maybe_transform(
                {"managed_account_allow_custom_pricing": managed_account_allow_custom_pricing},
                managed_account_update_params.ManagedAccountUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ManagedAccountUpdateResponse,
        )

    def list(
        self,
        *,
        filter: managed_account_list_params.Filter | Omit = omit,
        include_cancelled_accounts: bool | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: Literal["created_at", "email"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ManagedAccountListResponse, AsyncDefaultFlatPagination[ManagedAccountListResponse]]:
        """Lists the accounts managed by the current user.

        Users need to be explictly
        approved by Telnyx in order to become manager accounts.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[email][contains], filter[email][eq], filter[organization_name][contains],
              filter[organization_name][eq]

          include_cancelled_accounts: Specifies if cancelled accounts should be included in the results.

          sort: Specifies the sort order for results. By default sorting direction is ascending.
              To have the results sorted in descending order add the <code> -</code>
              prefix.<br/><br/> That is: <ul>

                <li>
                  <code>email</code>: sorts the result by the
                  <code>email</code> field in ascending order.
                </li>

                <li>
                  <code>-email</code>: sorts the result by the
                  <code>email</code> field in descending order.
                </li>
              </ul> <br/> If not given, results are sorted by <code>created_at</code> in descending order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/managed_accounts",
            page=AsyncDefaultFlatPagination[ManagedAccountListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "include_cancelled_accounts": include_cancelled_accounts,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    managed_account_list_params.ManagedAccountListParams,
                ),
            ),
            model=ManagedAccountListResponse,
        )

    async def get_allocatable_global_outbound_channels(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ManagedAccountGetAllocatableGlobalOutboundChannelsResponse:
        """
        Display information about allocatable global outbound channels for the current
        user. Only usable by account managers.
        """
        return await self._get(
            "/managed_accounts/allocatable_global_outbound_channels",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ManagedAccountGetAllocatableGlobalOutboundChannelsResponse,
        )

    async def update_global_channel_limit(
        self,
        id: str,
        *,
        channel_limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ManagedAccountUpdateGlobalChannelLimitResponse:
        """
        Update the amount of allocatable global outbound channels allocated to a
        specific managed account.

        Args:
          channel_limit: Integer value that indicates the number of allocatable global outbound channels
              that should be allocated to the managed account. Must be 0 or more. If the value
              is 0 then the account will have no usable channels and will not be able to
              perform outbound calling.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/managed_accounts/{id}/update_global_channel_limit",
            body=await async_maybe_transform(
                {"channel_limit": channel_limit},
                managed_account_update_global_channel_limit_params.ManagedAccountUpdateGlobalChannelLimitParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ManagedAccountUpdateGlobalChannelLimitResponse,
        )


class ManagedAccountsResourceWithRawResponse:
    def __init__(self, managed_accounts: ManagedAccountsResource) -> None:
        self._managed_accounts = managed_accounts

        self.create = to_raw_response_wrapper(
            managed_accounts.create,
        )
        self.retrieve = to_raw_response_wrapper(
            managed_accounts.retrieve,
        )
        self.update = to_raw_response_wrapper(
            managed_accounts.update,
        )
        self.list = to_raw_response_wrapper(
            managed_accounts.list,
        )
        self.get_allocatable_global_outbound_channels = to_raw_response_wrapper(
            managed_accounts.get_allocatable_global_outbound_channels,
        )
        self.update_global_channel_limit = to_raw_response_wrapper(
            managed_accounts.update_global_channel_limit,
        )

    @cached_property
    def actions(self) -> ActionsResourceWithRawResponse:
        return ActionsResourceWithRawResponse(self._managed_accounts.actions)


class AsyncManagedAccountsResourceWithRawResponse:
    def __init__(self, managed_accounts: AsyncManagedAccountsResource) -> None:
        self._managed_accounts = managed_accounts

        self.create = async_to_raw_response_wrapper(
            managed_accounts.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            managed_accounts.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            managed_accounts.update,
        )
        self.list = async_to_raw_response_wrapper(
            managed_accounts.list,
        )
        self.get_allocatable_global_outbound_channels = async_to_raw_response_wrapper(
            managed_accounts.get_allocatable_global_outbound_channels,
        )
        self.update_global_channel_limit = async_to_raw_response_wrapper(
            managed_accounts.update_global_channel_limit,
        )

    @cached_property
    def actions(self) -> AsyncActionsResourceWithRawResponse:
        return AsyncActionsResourceWithRawResponse(self._managed_accounts.actions)


class ManagedAccountsResourceWithStreamingResponse:
    def __init__(self, managed_accounts: ManagedAccountsResource) -> None:
        self._managed_accounts = managed_accounts

        self.create = to_streamed_response_wrapper(
            managed_accounts.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            managed_accounts.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            managed_accounts.update,
        )
        self.list = to_streamed_response_wrapper(
            managed_accounts.list,
        )
        self.get_allocatable_global_outbound_channels = to_streamed_response_wrapper(
            managed_accounts.get_allocatable_global_outbound_channels,
        )
        self.update_global_channel_limit = to_streamed_response_wrapper(
            managed_accounts.update_global_channel_limit,
        )

    @cached_property
    def actions(self) -> ActionsResourceWithStreamingResponse:
        return ActionsResourceWithStreamingResponse(self._managed_accounts.actions)


class AsyncManagedAccountsResourceWithStreamingResponse:
    def __init__(self, managed_accounts: AsyncManagedAccountsResource) -> None:
        self._managed_accounts = managed_accounts

        self.create = async_to_streamed_response_wrapper(
            managed_accounts.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            managed_accounts.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            managed_accounts.update,
        )
        self.list = async_to_streamed_response_wrapper(
            managed_accounts.list,
        )
        self.get_allocatable_global_outbound_channels = async_to_streamed_response_wrapper(
            managed_accounts.get_allocatable_global_outbound_channels,
        )
        self.update_global_channel_limit = async_to_streamed_response_wrapper(
            managed_accounts.update_global_channel_limit,
        )

    @cached_property
    def actions(self) -> AsyncActionsResourceWithStreamingResponse:
        return AsyncActionsResourceWithStreamingResponse(self._managed_accounts.actions)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from .operator_connect import (
    OperatorConnectResource,
    AsyncOperatorConnectResource,
    OperatorConnectResourceWithRawResponse,
    AsyncOperatorConnectResourceWithRawResponse,
    OperatorConnectResourceWithStreamingResponse,
    AsyncOperatorConnectResourceWithStreamingResponse,
)

__all__ = [
    "ActionsResource",
    "AsyncActionsResource",
    "ActionsResourceWithRawResponse",
    "AsyncActionsResourceWithRawResponse",
    "ActionsResourceWithStreamingResponse",
    "AsyncActionsResourceWithStreamingResponse",
    "OperatorConnectResource",
    "AsyncOperatorConnectResource",
    "OperatorConnectResourceWithRawResponse",
    "AsyncOperatorConnectResourceWithRawResponse",
    "OperatorConnectResourceWithStreamingResponse",
    "AsyncOperatorConnectResourceWithStreamingResponse",
]
